#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from onf_website.component.extfile.interfaces import IExtFileContainerTarget, IExtFileContainer, \
    IExtFileLinksContainerTarget, IExtFileLinksContainer
from pyams_file.interfaces import IFileInfo, IFile, IImage
from pyams_i18n.interfaces import II18n
from pyams_skin.interfaces import IInnerPage, IPageHeader
from pyams_skin.layer import IPyAMSLayer
from pyams_utils.interfaces.data import IObjectData
from pyams_zmi.interfaces.menu import IPropertiesMenu
from pyams_zmi.layer import IAdminLayer
from z3c.table.interfaces import IValues, IColumn

# import packages
from onf_website.component.extfile.zmi.widget import ExtFileLinkSelectFieldWidget
from onf_website.shared.common.zmi import WfModifiedContentColumnMixin
from pyams_form.form import AJAXEditForm
from pyams_form.security import ProtectedFormObjectMixin
from pyams_pagelet.pagelet import pagelet_config
from pyams_skin.page import DefaultPageHeaderAdapter
from pyams_skin.table import BaseTable, I18nColumn, TrashColumn
from pyams_skin.viewlet.menu import MenuItem, MenuDivider
from pyams_template.template import template_config
from pyams_utils.adapter import adapter_config, ContextRequestViewAdapter
from pyams_utils.size import get_human_size
from pyams_utils.url import absolute_url
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.form import AdminDialogEditForm
from pyams_zmi.view import AdminView
from pyramid.decorator import reify
from pyramid.view import view_config
from z3c.form import field
from z3c.table.column import GetAttrColumn
from zope.interface import implementer, alsoProvides

from onf_website import _


@viewlet_config(name='external-files.divider', context=IExtFileContainerTarget, layer=IAdminLayer,
                manager=IPropertiesMenu, permission='system.view', weight=299)
class ExtFileContainerMenuDivider(MenuDivider):
    """External files container menu divider"""


@viewlet_config(name='external-files.menu', context=IExtFileContainerTarget, layer=IAdminLayer,
                manager=IPropertiesMenu, permission='system.view', weight=300)
class ExtFileContainerMenu(MenuItem):
    """External files container menu"""

    label = _("External files...")
    icon_class = 'fa-file-text-o'
    url = '#external-files.html'


#
# External files container views
#

@pagelet_config(name='external-files.html', context=IExtFileContainerTarget, layer=IPyAMSLayer,
                permission='system.view')
@template_config(template='templates/container.pt', layer=IPyAMSLayer)
@implementer(IInnerPage)
class ExtFileContainerView(AdminView):
    """External files container view"""

    title = _("External files list")

    def __init__(self, context, request):
        super(ExtFileContainerView, self).__init__(context, request)
        self.files_table = ExtFileContainerTable(context, request, self, _("External files"), 'files')
        self.images_table = ExtFileContainerTable(context, request, self, _("Images"), 'images')
        self.videos_table = ExtFileContainerTable(context, request, self, _("Videos"), 'videos')
        self.audios_table = ExtFileContainerTable(context, request, self, _("Sounds"), 'audios')

    def update(self):
        super(ExtFileContainerView, self).update()
        self.files_table.update()
        self.images_table.update()
        self.videos_table.update()
        self.audios_table.update()


class ExtFileContainerTable(BaseTable):
    """External files container table"""

    hide_toolbar = True
    cssClasses = {'table': 'table table-bordered table-striped table-hover table-tight'}

    def __init__(self, context, request, view, title, property):
        super(ExtFileContainerTable, self).__init__(context, request)
        self.view = view
        self.title = title
        self.files_property = property
        self.object_data = {'ams-widget-toggle-button': 'false'}
        alsoProvides(self, IObjectData)

    @property
    def data_attributes(self):
        attributes = super(ExtFileContainerTable, self).data_attributes
        attributes['table'] = {'data-ams-location': absolute_url(IExtFileContainer(self.context), self.request),
                               'data-ams-datatable-sort': 'false',
                               'data-ams-datatable-pagination': 'false'}
        return attributes

    @reify
    def values(self):
        return list(super(ExtFileContainerTable, self).values)

    def render(self):
        if not self.values:
            if self.files_property == 'files':
                for table in (self.view.images_table, self.view.videos_table, self.view.audios_table):
                    if table.values:
                        return ''
                translate = self.request.localizer.translate
                return translate(_("No currently stored external file."))
            else:
                return ''
        return super(ExtFileContainerTable, self).render()


@adapter_config(name='name', context=(IExtFileContainerTarget, IPyAMSLayer, ExtFileContainerTable), provides=IColumn)
class ExtFileContainerNameColumn(I18nColumn, WfModifiedContentColumnMixin, GetAttrColumn):
    """External files container name column"""

    _header = _("Title")

    weight = 10

    def getValue(self, obj):
        return II18n(obj).query_attribute('title', request=self.request)


@adapter_config(name='filename', context=(IExtFileContainerTarget, IPyAMSLayer, ExtFileContainerTable), provides=IColumn)
class ExtFileContainerFilenameColumn(I18nColumn, GetAttrColumn):
    """External file container filename column"""

    _header = _("Filename")

    weight = 15

    def getValue(self, obj):
        data = II18n(obj).query_attribute('data', request=self.request)
        if data is not None:
            return IFileInfo(data).filename
        else:
            return '--'


@adapter_config(name='filesize', context=(IExtFileContainerTarget, IPyAMSLayer, ExtFileContainerTable), provides=IColumn)
class ExtFileContainerFileSizeColumn(I18nColumn, GetAttrColumn):
    """External file container file size column"""

    _header = _("Size")

    weight = 20

    def getValue(self, obj):
        data = II18n(obj).query_attribute('data', request=self.request)
        if data is not None:
            result = get_human_size(IFile(data).get_size())
            if IImage.providedBy(data):
                result = '{0} ({1[0]}x{1[1]})'.format(result, IImage(data).get_image_size())
            return result
        else:
            return 'N/A'


@adapter_config(name='trash', context=(IExtFileContainerTarget, IPyAMSLayer, ExtFileContainerTable), provides=IColumn)
class ExtFileContainerTrashColumn(ProtectedFormObjectMixin, TrashColumn):
    """External files container trash column"""


@adapter_config(context=(IExtFileContainerTarget, IPyAMSLayer, ExtFileContainerTable), provides=IValues)
class ExtFileContainerValues(ContextRequestViewAdapter):
    """External files container values"""

    @property
    def values(self):
        return getattr(IExtFileContainer(self.context), self.view.files_property)


@adapter_config(context=(IExtFileContainerTarget, IPyAMSLayer, ExtFileContainerView), provides=IPageHeader)
class ExtFileHeaderAdapter(DefaultPageHeaderAdapter):
    """External files container header adapter"""

    back_url = '#properties.html'
    icon_class = 'fa fa-fw fa-file-text-o'


#
# External files links edit form
#

@pagelet_config(name='extfile-links.html', context=IExtFileLinksContainerTarget, layer=IPyAMSLayer, permission='system.view')
class ExtFileLinksContainerLinksEditForm(AdminDialogEditForm):
    """External file links container edit form"""

    legend = _("Edit external files links")

    fields = field.Fields(IExtFileLinksContainer)
    fields['files'].widgetFactory = ExtFileLinkSelectFieldWidget

    ajax_handler = 'extfile-links.json'
    edit_permission = 'onf.ManageContent'


@view_config(name='extfile-links.json', context=IExtFileLinksContainerTarget, request_type=IPyAMSLayer,
             permission='onf.ManageContent', renderer='json', xhr=True)
class ExtFileLinksContainerAJAXEditForm(AJAXEditForm, ExtFileLinksContainerLinksEditForm):
    """External file links container edit form, JSON renderer"""
