#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
from pyams_skin.help import ContentHelp

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from onf_website.component.hearing.interfaces import IHearingTarget, IHearingInfo
from pyams_form.interfaces.form import IWidgetForm, IInnerSubForm
from pyams_skin.interfaces import IInnerPage, IPageHeader, IContentHelp
from pyams_skin.layer import IPyAMSLayer
from pyams_zmi.interfaces.menu import IPropertiesMenu
from pyams_zmi.layer import IAdminLayer
from z3c.form.interfaces import INPUT_MODE

# import packages
from onf_website.shared.common.zmi import WfSharedContentHeaderAdapter
from pyams_form.form import AJAXEditForm, InnerEditForm
from pyams_form.group import NamedWidgetsGroup
from pyams_form.schema import ResetButton
from pyams_pagelet.pagelet import pagelet_config
from pyams_skin.viewlet.menu import MenuItem
from pyams_utils.adapter import adapter_config
from pyams_utils.property import cached_property
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.form import AdminEditForm
from pyramid.view import view_config
from z3c.form import field, button
from z3c.form.browser.checkbox import SingleCheckBoxFieldWidget
from zope.interface import implementer, alsoProvides, Interface

from onf_website import _


@viewlet_config(name='hearing.menu', context=IHearingTarget, layer=IAdminLayer,
                manager=IPropertiesMenu, permission='system.view', weight=210)
class HearingMenu(MenuItem):
    """Hearing menu"""

    label = _("Hearing...")
    icon_class = 'fa-volume-up'
    url = '#hearing.html'


class IHearingEditGroup(Interface):
    """Inner hearing edit group"""
    
    
class IHearingEditButtons(Interface):
    """Hearing form buttons"""

    reset = ResetButton(name='reset', title=_("Reset"))
    submit = button.Button(name='submit', title=_("Submit"))


@pagelet_config(name='hearing.html', context=IHearingTarget, layer=IPyAMSLayer, permission='system.view')
@implementer(IWidgetForm, IInnerPage)
class HearingEditForm(AdminEditForm):
    """Hearing edit form"""

    legend = _("Content hearing")

    @property
    def fields(self):
        if IHearingInfo(self.context).can_inherit:
            fields = field.Fields(IHearingInfo).select('no_inherit')
        else:
            fields = field.Fields(Interface)
        return fields

    @property
    def buttons(self):
        if self.mode == INPUT_MODE:
            return button.Buttons(IHearingEditButtons)
        else:
            return button.Buttons(Interface)

    ajax_handler = 'hearing.json'

    def update(self):
        super(HearingEditForm, self).update()
        if IHearingInfo(self.context).can_inherit:
            group = NamedWidgetsGroup(self, 'hearing', self.widgets,
                                      ('no_inherit', ),
                                      legend=_("Don't inherit parent hearing"),
                                      css_class='inner',
                                      switch=True,
                                      checkbox_switch=True,
                                      checkbox_field=IHearingInfo['no_inherit'])
        else:
            group = NamedWidgetsGroup(self, 'hearing', self.widgets, (), css_class='inner')
        alsoProvides(group, IHearingEditGroup)
        [subform.update() for subform in group.subforms]
        self.add_group(group)


@view_config(name='hearing.json', context=IHearingTarget, request_type=IPyAMSLayer,
             permission='onf.ManageContent', renderer='json', xhr=True)
class HearingAJAXEditForm(AJAXEditForm, HearingEditForm):
    """Hearing edit form, JSON renderer"""

    @cached_property
    def forms(self):
        if IHearingInfo(self.context).can_inherit:
            group = NamedWidgetsGroup(self, 'hearing', self.widgets,
                                      ('no_inherit', ),
                                      legend=_("Don't inherit parent hearing"),
                                      css_class='inner',
                                      switch=True,
                                      checkbox_switch=True,
                                      checkbox_field=IHearingInfo['no_inherit'])
        else:
            group = NamedWidgetsGroup(self, 'hearing', self.widgets, (), css_class='inner')
        alsoProvides(group, IHearingEditGroup)
        [subform.update() for subform in group.subforms]
        self.add_group(group)
        return super(HearingAJAXEditForm, self).forms + group.subforms


@adapter_config(name='targets', context=(IHearingTarget, IPyAMSLayer, IHearingEditGroup), provides=IInnerSubForm)
class HearingMainEditSubform(InnerEditForm):
    """Hearing targets edit form"""

    legend = None
    fields = field.Fields(IHearingInfo).select('targets')
    weight = 1

    def __new__(cls, context, request, view):
        if not context.handle_hearing_targets:
            return None
        return InnerEditForm.__new__(cls)

    def update(self):
        super(HearingMainEditSubform, self).update()
        self.add_group(NamedWidgetsGroup(self, 'targets', self.widgets, ('targets', ),
                                         legend=_("Selected targets"),
                                         css_class='inner',
                                         switch=False))


@adapter_config(name='privacy', context=(IHearingTarget, IPyAMSLayer, IHearingEditGroup), provides=IInnerSubForm)
class HearingPrivacyEditSubform(InnerEditForm):
    """Hearing privacy edit form"""

    legend = None
    fields = field.Fields(IHearingInfo).select('private', 'private_groups')
    fields['private'].widgetFactory = SingleCheckBoxFieldWidget
    weight = 2

    def __new__(cls, context, request, view):
        if not context.handle_hearing_privacy:
            return None
        return InnerEditForm.__new__(cls)

    def update(self):
        super(HearingPrivacyEditSubform, self).update()
        self.add_group(NamedWidgetsGroup(self, 'privacy', self.widgets,
                                         ('private', 'private_groups'),
                                         legend=_("Restrict access"),
                                         css_class='inner',
                                         switch=True,
                                         checkbox_switch=True,
                                         checkbox_field=IHearingInfo['private']))


@adapter_config(name='source', context=(IHearingTarget, IPyAMSLayer, IHearingEditGroup), provides=IInnerSubForm)
class HearingSourceEditSubform(InnerEditForm):
    """Hearing source edit form"""

    legend = None
    fields = field.Fields(IHearingInfo).select('source', 'diffusion_sites')
    weight = 3

    def __new__(cls, context, request, view):
        if not context.handle_hearing_source:
            return None
        return InnerEditForm.__new__(cls)

    def update(self):
        super(HearingSourceEditSubform, self).update()
        self.add_group(NamedWidgetsGroup(self, 'source', self.widgets, ('source', 'diffusion_sites'),
                                         legend=_("Diffusion sites"),
                                         css_class='inner',
                                         switch=False))


@adapter_config(name='scope', context=(IHearingTarget, IPyAMSLayer, IHearingEditGroup), provides=IInnerSubForm)
class HearingScopeEditSubform(InnerEditForm):
    """Hearing scope edit form"""

    legend = None
    fields = field.Fields(IHearingInfo).select('ignore_location', 'departments', 'departments_forests',
                                               'domanials_only', 'communals_only', 'forests')
    fields['ignore_location'].widgetFactory = SingleCheckBoxFieldWidget
    fields['departments_forests'].widgetFactory = SingleCheckBoxFieldWidget
    fields['domanials_only'].widgetFactory = SingleCheckBoxFieldWidget
    fields['communals_only'].widgetFactory = SingleCheckBoxFieldWidget
    weight = 4

    def __new__(cls, context, request, view):
        if not context.handle_hearing_scope:
            return None
        return InnerEditForm.__new__(cls)

    def update(self):
        super(HearingScopeEditSubform, self).update()
        self.add_group(NamedWidgetsGroup(self, 'scope', self.widgets,
                                         ('ignore_location', 'departments', 'departments_forests',
                                          'domanials_only', 'communals_only', 'forests'),
                                         legend=_("Override location settings"),
                                         css_class='inner',
                                         switch=True,
                                         checkbox_switch=True,
                                         checkbox_field=IHearingInfo['ignore_location']))

    def updateWidgets(self, prefix=None):
        super(HearingScopeEditSubform, self).updateWidgets(prefix)
        self.widgets['domanials_only'].label_css_class = 'input padding-left-20'
        self.widgets['communals_only'].label_css_class = 'input padding-left-20'


@adapter_config(context=(IHearingTarget, IPyAMSLayer, HearingScopeEditSubform), provides=IContentHelp)
class HearingScopeEditSubformHelpAdapter(ContentHelp):
    """Scope edit subform help adapter"""

    outer_margin = 10
    message = _("By default, hearing scope is based on location settings. You can check this box to "
                "override location and set custom settings.")
    message_format = 'text'


@adapter_config(context=(IHearingTarget, IAdminLayer, HearingEditForm), provides=IPageHeader)
class HearingHeaderAdapter(WfSharedContentHeaderAdapter):
    """Shared content hearing header adapter"""

    icon_class = 'fa fa-fw fa-volume-up'
