#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
from pyramid.view import view_config
from pyramid.decorator import reify
from z3c.table.column import GetAttrColumn
from z3c.table.interfaces import IColumn, IValues
from onf_website.component.links.interfaces import ILinkContainerTarget, ILinkContainer, IInternalLink, \
    ILinkLinksContainerTarget, ILinkLinksContainer
from onf_website.component.links.zmi.widget import LinkLinkSelectFieldWidget
from onf_website.shared.common.zmi import WfModifiedContentColumnMixin
from pyams_form.form import AJAXEditForm
from pyams_form.security import ProtectedFormObjectMixin
from pyams_i18n.interfaces import II18n
from pyams_pagelet.pagelet import pagelet_config
from pyams_sequence.utility import get_sequence_dict
from pyams_skin.interfaces import IInnerPage, IPageHeader
from pyams_skin.layer import IPyAMSLayer
from pyams_skin.page import DefaultPageHeaderAdapter
from pyams_skin.table import BaseTable, I18nColumn, TrashColumn
from pyams_skin.viewlet.menu import MenuItem
from pyams_template.template import template_config
from pyams_utils.adapter import adapter_config, ContextRequestViewAdapter
from pyams_utils.interfaces.data import IObjectData
from pyams_utils.url import absolute_url
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.form import AdminDialogEditForm
from pyams_zmi.interfaces.menu import IPropertiesMenu
from pyams_zmi.layer import IAdminLayer
from pyams_zmi.view import AdminView

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces

# import packages
from z3c.form import field
from zope.interface import implementer, alsoProvides

from onf_website import _


@viewlet_config(name='links.menu', context=ILinkContainerTarget, layer=IAdminLayer,
                manager=IPropertiesMenu, permission='system.view', weight=310)
class LinkContainerMenu(MenuItem):
    """Links container menu"""

    label = _("Useful links...")
    icon_class = 'fa-link'
    url = '#links.html'


#
# Links container views
#

@pagelet_config(name='links.html', context=ILinkContainerTarget, layer=IPyAMSLayer,
                permission='system.view')
@template_config(template='templates/container.pt', layer=IPyAMSLayer)
@implementer(IInnerPage)
class LinkContainerView(AdminView):
    """Links container view"""

    title = _("Useful links list")

    def __init__(self, context, request):
        super(LinkContainerView, self).__init__(context, request)
        self.links_table = LinkContainerTable(context, request)

    def update(self):
        super(LinkContainerView, self).update()
        self.links_table.update()


class LinkContainerTable(BaseTable):
    """Links container table"""

    hide_header = True
    cssClasses = {'table': 'table table-bordered table-striped table-hover table-tight'}

    def __init__(self, context, request):
        super(LinkContainerTable, self).__init__(context, request)
        self.object_data = {'ams-widget-toggle-button': 'false'}
        alsoProvides(self, IObjectData)

    @property
    def data_attributes(self):
        attributes = super(LinkContainerTable, self).data_attributes
        attributes['table'] = {'data-ams-location': absolute_url(ILinkContainer(self.context), self.request),
                               'data-ams-datatable-sort': 'false',
                               'data-ams-datatable-pagination': 'false'}
        return attributes

    @reify
    def values(self):
        return list(super(LinkContainerTable, self).values)

    def render(self):
        if not self.values:
            translate = self.request.localizer.translate
            return translate(_("No currently defined link."))
        return super(LinkContainerTable, self).render()


@adapter_config(name='name', context=(ILinkContainerTarget, IPyAMSLayer, LinkContainerTable), provides=IColumn)
class LinkContainerNameColumn(I18nColumn, WfModifiedContentColumnMixin, GetAttrColumn):
    """Links container name column"""

    _header = _("Title")

    weight = 10

    def getValue(self, obj):
        return II18n(obj).query_attribute('title', request=self.request)


@adapter_config(name='target', context=(ILinkContainerTarget, IPyAMSLayer, LinkContainerTable), provides=IColumn)
class LinkContainerTargetColumn(I18nColumn, GetAttrColumn):
    """Links container target column"""

    _header = _("Link target")

    weight = 20

    def getValue(self, obj):
        if IInternalLink.providedBy(obj):
            mapping = get_sequence_dict(obj.get_target())
            return mapping['text']
        else:
            return obj.url


@adapter_config(name='trash', context=(ILinkContainerTarget, IPyAMSLayer, LinkContainerTable), provides=IColumn)
class LinkContainerTrashColumn(ProtectedFormObjectMixin, TrashColumn):
    """Links container trash column"""


@adapter_config(context=(ILinkContainerTarget, IPyAMSLayer, LinkContainerTable), provides=IValues)
class LinkContainerValues(ContextRequestViewAdapter):
    """Links container values"""

    @property
    def values(self):
        return ILinkContainer(self.context).values()


@adapter_config(context=(ILinkContainerTarget, IPyAMSLayer, LinkContainerView), provides=IPageHeader)
class LinkHeaderAdapter(DefaultPageHeaderAdapter):
    """Links container header adapter"""

    back_url = '#properties.html'
    icon_class = 'fa fa-fw fa-link'


#
# Links links edit form
#

@pagelet_config(name='link-links.html', context=ILinkLinksContainerTarget, layer=IPyAMSLayer,
                permission='system.view')
class LinkLinksContainerLinksEditForm(AdminDialogEditForm):
    """Links links container edit form"""

    legend = _("Edit useful links links")

    fields = field.Fields(ILinkLinksContainer)
    fields['links'].widgetFactory = LinkLinkSelectFieldWidget

    ajax_handler = 'link-links.json'
    edit_permission = 'onf.ManageContent'


@view_config(name='link-links.json', context=ILinkLinksContainerTarget, request_type=IPyAMSLayer,
             permission='onf.ManageContent', renderer='json', xhr=True)
class LinkLinksContainerAJAXEditForm(AJAXEditForm, LinkLinksContainerLinksEditForm):
    """Links links container edit form, JSON renderer"""
